package main

import (
	"fmt"
	"log"
	"os"
	streamserver "stream-server"

	"github.com/urfave/cli"

	"stream-server/internal/server"
	"stream-server/internal/util"
)

const docURL = "https://stream-server.h-da.io"

func main() {

	app := cli.NewApp()

	var port int
	var tlsport int
	var tlsCert, tlsKey string

	var listenAddr string
	var flagUseHostNamespace bool
	var flagHelp bool

	app.Usage = docURL
	app.HideHelp = true

	app.Version = streamserver.Version

	app.Flags = []cli.Flag{
		cli.IntFlag{
			Name:        "port",
			Value:       8080,
			Usage:       "http port to listen on",
			Destination: &port,
		},
		cli.IntFlag{
			Name:        "tlsport",
			Value:       8443,
			Usage:       "https port to listen on",
			Destination: &tlsport,
		},
		cli.StringFlag{
			Name:        "listen",
			Value:       "0.0.0.0",
			Usage:       "address to listen on",
			Destination: &listenAddr,
		},
		cli.StringFlag{
			Name:        "tlscert",
			Usage:       "TLS cert",
			Destination: &tlsCert,
		},
		cli.StringFlag{
			Name:        "tlskey",
			Usage:       "TLS key",
			Destination: &tlsKey,
		},
		cli.BoolFlag{
			Name:        "use-host-namespace",
			Usage:       "use the Host header as namespace",
			Destination: &flagUseHostNamespace,
		},
		cli.BoolFlag{
			Name:        "help, h",
			Usage:       "show help",
			Destination: &flagHelp,
		},
	}

	app.Action = func(c *cli.Context) error {
		if flagHelp {
			cli.ShowAppHelp(c)
			return nil
		}

		if len(c.Args()) != 0 { // we had arguments. TODO: can this be disabled somehow in cli?
			return fmt.Errorf("Unknown arguments: %v", c.Args())
		}

		withTLS := tlsCert != "" && tlsKey != ""

		fmt.Print("Serving on\n")
		if listenAddr == "0.0.0.0" {
			fmt.Printf("  http://%s:%d\n", "localhost", port)
			if withTLS {
				fmt.Printf("  https://%s:%d\n", "localhost", tlsport)
			}

			if outboundIP, err := util.GetOutboundIP(); err == nil {
				fmt.Printf("  http://%s:%d\n", outboundIP, port)
				if withTLS {
					fmt.Printf("  https://%s:%d\n", outboundIP, tlsport)
				}
			}
		} else {
			fmt.Printf("  http://%s:%d\n", listenAddr, port)
			if withTLS {
				fmt.Printf("  https://%s:%d\n", listenAddr, tlsport)
			}
		}

		fmt.Printf("Read the quickstart at %s/quickstart to get started.\n", docURL)
		fmt.Printf("Stop with Ctrl-C or close this terminal.\n")

		router := server.GetRouter(flagUseHostNamespace)

		for {
			errs := make(chan error)

			go func() {
				if err := server.ServeHTTP(fmt.Sprintf("%s:%d", listenAddr, port), router); err != nil {
					errs <- err
				}
			}()

			if withTLS {
				go func() {
					if err := server.ServeHTTPS(fmt.Sprintf("%s:%d", listenAddr, tlsport), router, tlsCert, tlsKey); err != nil {
						errs <- err
					}
				}()
			}

			select {
			case err := <-errs:
				return err
			}
		}

	}

	err := app.Run(os.Args)
	if err != nil {
		log.Fatal(err)
	}
}
