package com.example.myapplication;

import android.net.Uri;
import android.text.TextUtils;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class StreamServerClient {
    private Uri baseUri;

    /**
     * Creates a new instance of StreamServerClient.
     * The instance is very lightweight so it is okay to create a new instance for each request.
     *
     * @param ipAddressWithPort The IP address with port e.g. "10.0.2.2:8080"
     */
    public StreamServerClient(String ipAddressWithPort) {
        baseUri = Uri.parse("http://" + ipAddressWithPort);
    }

    /**
     * Creates a new instance of StreamServerClient.
     * The instance is very lightweight so it is okay to create a new instance for each request.
     *
     * @param ipAddress The IP address e.g. "10.0.2.2"
     * @param port The port e.g. 8080
     */
    public StreamServerClient(String ipAddress, int port) {
        baseUri = Uri.parse("http://" + ipAddress + ":" + port);
    }

    private String doRequest(String path, @Nullable Map<String, String> params) throws IOException {
        Uri.Builder builder = baseUri.buildUpon().appendEncodedPath(path.substring(1));

        if (params != null) {
            for (Map.Entry<String, String> entry : params.entrySet()) {
                builder.appendQueryParameter(entry.getKey(), entry.getValue());
            }
        }

        URL url = new URL(builder.build().toString());

        HttpURLConnection conn = (HttpURLConnection) url.openConnection();

        try {
            conn.connect();
            int status = conn.getResponseCode();

            BufferedReader br = new BufferedReader(new InputStreamReader(conn.getInputStream()));
            StringBuilder sb = new StringBuilder();
            String line;
            while ((line = br.readLine()) != null) {
                sb.append(line);
                sb.append("\n"); // add the stripped line back
            }
            br.close();

            String content = sb.toString();

            if (status != HttpURLConnection.HTTP_OK) { // status != 200
                throw new IOException(String.format("HTTP request %s failed with status %d and content: %s", conn, status, content));
            }

            return content;
        } finally {
            conn.disconnect();
        }
    }

    private List<JSONObject> jsonArrayToJsonObjectList(JSONArray array) throws JSONException {
        List<JSONObject> list = new ArrayList<>();

        for (int i = 0; i < array.length(); i++) {
            list.add(array.getJSONObject(i));
        }

        return list;
    }

    /**
     * Updates and returns the new state of the display.
     * Reference at <a href="https://stream-server.h-da.io/reference#display">https://stream-server.h-da.io/reference#display</a>
     *
     * @param large_channel Sets the channel name of the large screen.
     *                      This screen is the always visible main screen.
     *                      Set to "" to disable. Set to null to keep the old value.
     *
     * @param small_channel Sets the channel name of the small screen.
     *                      This screen is a small Picture-in-Picture like screen in the top right corner.
     *                      Set to "" to disable. Set to null to keep the old value.
     *
     * @param volume Sets the volume of the large screen.
     *               Set to 0 or 0.0 to mute.
     *               Values are in the range of 0.0 - 1.0.
     *               Note: The small screen is always muted.
     *               Set to null to keep the old value.
     *
     * @param small_scale Sets the scale (size) of the small screen.
     *                    The value is relative to the width of the large screen.
     *                    Reasonable values are in the range of 0.2 - 0.5.
     *                    It is best to just experiment with this setting.
     *                    Set to null to keep the old value.
     *
     * @param show_chat Sets whether the chat should be shown or not.
     *                  Set to null to keep the old value.
     */
    public JSONObject display(@Nullable String large_channel, @Nullable String small_channel, @Nullable Float volume, @Nullable Float small_scale, @Nullable Boolean show_chat) throws IOException, JSONException {
        Map<String, String> params = new HashMap<>();
        if (large_channel != null) {
            params.put("large_channel", large_channel);
        }

        if (small_channel != null) {
            params.put("small_channel", small_channel);
        }

        if (volume != null) {
            params.put("volume", volume.toString());
        }

        if (small_scale != null) {
            params.put("small_scale", small_scale.toString());
        }

        if (show_chat != null) {
            params.put("show_chat", show_chat.toString());
        }

        String content = doRequest("/display", params);
        return new JSONObject(content);
    }

    /**
     * Gets the most popular games on Twitch right now.
     * Reference at <a href="https://stream-server.h-da.io/reference#twitchgettopgames">https://stream-server.h-da.io/reference#twitchgamestop</a>
     */
    public List<JSONObject> twitchGetTopGames() throws IOException, JSONException {
        String content = doRequest("/twitch/getTopGames", null);

        return jsonArrayToJsonObjectList(new JSONArray(content));
    }

    /**
     * Searches games by query.
     * Reference at <a href="https://stream-server.h-da.io/reference#twitchsearchgames">https://stream-server.h-da.io/reference#twitchsearchgames</a>
     *
     * @param query What should be searched.
     */
    public List<JSONObject> twitchSearchGames(@NonNull String query) throws IOException, JSONException {
        Map<String, String> params = new HashMap<>();
        params.put("query", query);

        String content = doRequest("/twitch/searchGames", params);

        return jsonArrayToJsonObjectList(new JSONArray(content));
    }

    /**
     * Gets the most popular streams on Twitch right now. Optionally filter by channels, game and language.
     * Reference at <a href="https://stream-server.h-da.io/reference#twitchgettopstreams">https://stream-server.h-da.io/reference#twitchgettopstreams</a>
     *
     * @param channels Specify up to 100 channels separated by `,` that the search should be limited to.
     *                 See the examples in the reference why that would be useful.
     *
     * @param game Specify a game that the search should be limited to.
     *
     * @param language Specify a language that the search should be limited to.
     */
    public List<JSONObject> twitchGetTopStreams(@Nullable String[] channels, @Nullable String game, @Nullable String language) throws IOException, JSONException {
        Map<String, String> params = new HashMap<>();
        if (channels != null) {
            params.put("channels", TextUtils.join(",", channels));
        }

        if (game != null) {
            params.put("game", game);
        }

        if (language != null) {
            params.put("language", language);
        }

        String content = doRequest("/twitch/getTopStreams", params);

        return jsonArrayToJsonObjectList(new JSONArray(content));
    }

    /**
     * Gets the featured games on Twitch's homepage.
     * Reference at <a href="https://stream-server.h-da.io/reference#twitchgetfeaturedstreams">https://stream-server.h-da.io/reference#twitchgetfeaturedstreams</a>
     */
    public List<JSONObject> twitchGetFeaturedStreams() throws IOException, JSONException {
        String content = doRequest("/twitch/getFeaturedStreams", null);

        return jsonArrayToJsonObjectList(new JSONArray(content));
    }
}
