package server

import (
	"net/http"
	"os"
	streamserver "stream-server"

	"stream-server/internal/util"
	"stream-server/pkg/twitch"
)

// client is the Twitch Client instance
var client *twitch.Client

func init() {
	// If you want to use your own twitch client id set the TWITCH_CLIENT_ID environment variable.

	if twitchClientID := os.Getenv("TWITCH_CLIENT_ID"); twitchClientID != "" {
		client = twitch.NewClient(twitchClientID)
	} else {
		client = twitch.NewClient(streamserver.DefaultTwitchClientID)
	}
}

// twitchGetTopGamesHandler handles the GetTopGames endpoint
func twitchGetTopGamesHandler(w http.ResponseWriter, r *http.Request) {
	response, err := client.GetTopGames()
	if err != nil {
		http.Error(w, err.Error(), http.StatusBadGateway)
		return
	}

	util.ServeIndentedJSON(w, r, &response)
}

// twitchSearchGamesHandler handles the SearchGames endpoint
func twitchSearchGamesHandler(w http.ResponseWriter, r *http.Request) {
	paramQuery, err := util.GetSingleQueryParam(r, "query")
	if err != nil {
		http.Error(w, err.Error(), http.StatusBadRequest)
		return
	}

	if paramQuery == nil || *paramQuery == "" {
		http.Error(w, "Parameter 'query' is required and can not be empty.", http.StatusBadRequest)
		return
	}

	response, err := client.SearchGames(*paramQuery)
	if err != nil {
		http.Error(w, err.Error(), http.StatusBadGateway)
		return
	}

	util.ServeIndentedJSON(w, r, &response)
}

// twitchGetTopStreamsHandler handles the GetTopStreams endpoint
func twitchGetTopStreamsHandler(w http.ResponseWriter, r *http.Request) {
	paramChannels := util.GetMultipleQueryParams(r, "channels")
	game, err := util.GetSingleQueryParam(r, "game")
	if err != nil {
		http.Error(w, err.Error(), http.StatusBadRequest)
		return
	}

	language, err := util.GetSingleQueryParam(r, "language")
	if err != nil {
		http.Error(w, err.Error(), http.StatusBadRequest)
		return
	}

	response, err := client.GetTopStreams(paramChannels, game, language)
	if err != nil {
		http.Error(w, err.Error(), http.StatusBadGateway)
		return
	}

	util.ServeIndentedJSON(w, r, &response)
}

// twitchGetFeaturedStreamsHandler handles the GetFeaturedStreams endpoint
func twitchGetFeaturedStreamsHandler(w http.ResponseWriter, r *http.Request) {
	response, err := client.GetFeaturedStreams()
	if err != nil {
		http.Error(w, err.Error(), http.StatusBadGateway)
		return
	}

	util.ServeIndentedJSON(w, r, &response)
}
