package util

import (
	"bytes"
	"encoding/json"
	"fmt"
	"net"
	"net/http"
	"os/exec"
	"runtime"
	"strings"
)

// ServeIndentedJSON marshals 'v' to JSON, automatically escaping HTML and setting the
// Content-Type as application/json.
// adapted from responder.JSON
func ServeIndentedJSON(w http.ResponseWriter, r *http.Request, v interface{}) {
	buf := &bytes.Buffer{}
	enc := json.NewEncoder(buf)
	enc.SetEscapeHTML(true)
	enc.SetIndent("", "\t")

	if err := enc.Encode(v); err != nil {
		http.Error(w, err.Error(), http.StatusInternalServerError)
		return
	}

	w.Header().Set("Content-Type", "application/json; charset=utf-8")
	w.Header().Set("Cache-Control", "no-store")
	w.WriteHeader(http.StatusOK)
	w.Write(buf.Bytes())
}

// GetSingleQueryParam is a helper function to get a single param from a request while making sure the param is present and has only a single value.
// If multiple values are present an error is returned
// If no values are found value is nil
func GetSingleQueryParam(r *http.Request, param string) (value *string, err error) {
	foundValues, found := r.URL.Query()[param]
	if !found {
		return
	}

	valuesCount := len(foundValues)

	if valuesCount == 1 {
		value = &foundValues[0]
		return
	}

	err = fmt.Errorf("Got %d values for parameter '%s' but expected only one", valuesCount, param)
	return
}

// GetMultipleQueryParams is a helper function to get multiple params from a request while making sure the param is present
// Both ?key=value1,value2,value3 and ?key=value1&key=value2&key=value3 are supported
// If no values are found values is nil
func GetMultipleQueryParams(r *http.Request, param string) (values []string) {
	foundValues, found := r.URL.Query()[param]
	if !found {
		return
	}

	values = make([]string, 0)

	for _, value := range foundValues {
		for _, splitValue := range strings.Split(value, ",") {
			if splitValue != "" {
				values = append(values, splitValue)
			}
		}
	}

	return
}

// GetOutboundIP returns the outbound IP of this device.
// It is adapted from https://stackoverflow.com/a/37382208/2607571
// It works by creating an UDP socket to an arbitrary IP and port.
// As UDP is connection-less it does not send any packets until a Send call.
// This way we can get the local address without creating any connections.
func GetOutboundIP() (net.IP, error) {
	conn, err := net.Dial("udp", "1.1.1.1:80")
	if err != nil {
		return nil, err
	}
	defer conn.Close()

	localAddr := conn.LocalAddr().(*net.UDPAddr)

	return localAddr.IP, nil
}

// OpenBrowser opens the system's default browser to the given url.
// Currently supported: linux, windows and darwin (macOS).
// From https://gist.github.com/hyg/9c4afcd91fe24316cbf0
func OpenBrowser(url string) (err error) {
	switch runtime.GOOS {
	case "linux":
		err = exec.Command("xdg-open", url).Start()
	case "windows":
		err = exec.Command("rundll32", "url.dll,FileProtocolHandler", url).Start()
	case "darwin":
		err = exec.Command("open", url).Start()
	default:
		err = fmt.Errorf("unsupported platform")
	}
	return
}

// Info: The following two definitions are legacy and are currently not used.

// ListeningAddress represents a IP address, its interface name and if it is a loopback address.
// It is used to display all addresses a socket is listening on.
type ListeningAddress struct {
	IP            net.IP
	InterfaceName string
	IsLoopback    bool
}

// GetListeningAddresses returns a list of listeningAddress that contains the local IP addresses.
func GetListeningAddresses() (addresses []ListeningAddress, err error) {

	ifaces, err := net.Interfaces()
	if err != nil {
		return nil, err
	}

	for _, iface := range ifaces {
		if iface.Flags&net.FlagUp == 0 {
			continue // interface down
		}

		addrs, err := iface.Addrs()
		if err != nil {
			continue // this interface does not support Addrs
		}

		for _, addr := range addrs {
			var ip net.IP
			switch v := addr.(type) {
			case *net.IPNet:
				ip = v.IP
			case *net.IPAddr:
				ip = v.IP
			}

			ip = ip.To4()
			if ip == nil {
				continue // not an ipv4 address
			}

			addresses = append(addresses, ListeningAddress{
				IP:            ip,
				InterfaceName: iface.Name,
				IsLoopback:    ip.IsLoopback(),
			})
		}
	}
	return
}
