package twitch

import "net/url"

// twitchGame is the structure for games that we get from twitch.
type twitchGame struct {
	Name string `json:"name"`
	Box  struct {
		Large string `json:"large"`
	} `json:"box"`

	Logo struct {
		Large string `json:"large"`
	} `json:"logo"`

	Popularity int `json:"popularity"`
}

// Game is the structure for games that we return to the user.
type Game struct {
	Name       string `json:"name"`
	Viewers    int    `json:"viewers"`
	BoxImgURL  string `json:"box_img_url"`
	LogoImgURL string `json:"logo_img_url"`
}

// toSimplified simplifies a twitchGame (that we got from twitch) to a Game (that we return to the user).
func (g *twitchGame) toSimplified() Game {
	return Game{
		Name:       g.Name,
		Viewers:    g.Popularity,
		BoxImgURL:  g.Box.Large,
		LogoImgURL: g.Logo.Large,
	}
}

// GetTopGames gets the top games from the twitch API.
func (c *Client) GetTopGames() (response []Game, err error) {
	var twitchResponse struct {
		Top []struct {
			Game twitchGame
		} `json:"top"`
	}

	err = c.Request("/games/top", url.Values{}, &twitchResponse)
	if err != nil {
		return
	}

	response = make([]Game, 0)

	for _, game := range twitchResponse.Top {
		response = append(response, game.Game.toSimplified())
	}

	return
}

// SearchGames searches games via the twitch API.
func (c *Client) SearchGames(query string) (response []Game, err error) {
	var twitchResponse struct {
		Games []twitchGame `json:"games"`
	}

	params := url.Values{}
	params.Set("query", query)

	err = c.Request("/search/games", params, &twitchResponse)
	if err != nil {
		return
	}

	response = make([]Game, 0)

	for _, game := range twitchResponse.Games {
		response = append(response, game.toSimplified())
	}

	return
}
