package twitch

import (
	"math"
	"net/url"
	"strings"
)

// twitchStream is the structure for streams that we get from twitch.
type twitchStream struct {
	AverageFPS float64 `json:"average_fps"`
	CreatedAt  string  `json:"created_at"`
	Game       string  `json:"game"`

	Preview struct {
		Large string `json:"large"`
	} `json:"preview"`

	VideoHeight int `json:"video_height"`
	Viewers     int `json:"viewers"`

	Channel struct {
		Status      string `json:"status"`
		Language    string `json:"language"`
		Mature      bool   `json:"mature"`
		Name        string `json:"name"`
		DisplayName string `json:"display_name"`
		Logo        string `json:"logo"`
	} `json:"channel"`
}

// Stream is the structure for streams that we return to the user.
type Stream struct {
	AverageFPS         int    `json:"average_fps"`
	StartedAt          string `json:"started_at"`
	Game               string `json:"game"`
	PreviewImgURL      string `json:"preview_img_url"`
	VideoHeight        int    `json:"video_height"`
	Viewers            int    `json:"viewers"`
	Status             string `json:"status"`
	Language           string `json:"language"`
	Mature             bool   `json:"mature"`
	ChannelName        string `json:"channel_name"`
	ChannelDisplayName string `json:"channel_display_name"`
	ChannelLogoImgURL  string `json:"channel_logo_img_url"`
}

// toSimplified simplifies a twitchStream (that we got from twitch) to a Stream (that we return to the user).
func (s *twitchStream) toSimplified() Stream {
	return Stream{
		AverageFPS:         int(math.Round(s.AverageFPS)),
		StartedAt:          s.CreatedAt,
		Game:               s.Game,
		PreviewImgURL:      s.Preview.Large,
		VideoHeight:        s.VideoHeight,
		Viewers:            s.Viewers,
		Status:             s.Channel.Status,
		Language:           s.Channel.Language,
		Mature:             s.Channel.Mature,
		ChannelName:        s.Channel.Name,
		ChannelDisplayName: s.Channel.DisplayName,
		ChannelLogoImgURL:  s.Channel.Logo,
	}
}

// GetFeaturedStreams gets the featured streams from the twitch API.
func (c *Client) GetFeaturedStreams() (response []Stream, err error) {
	var twitchResponse struct {
		Featured []struct {
			Stream twitchStream `json:"stream"`
		} `json:"featured"`
	}

	err = c.Request("/streams/featured", url.Values{}, &twitchResponse)
	if err != nil {
		return
	}

	response = make([]Stream, 0)

	for _, featured := range twitchResponse.Featured {
		if featured.Stream.Game != "" {
			response = append(response, featured.Stream.toSimplified())
		}
	}

	return
}

// GetTopStreams gets the top streams from the twitch API.
func (c *Client) GetTopStreams(channels []string, game *string, language *string) (response []Stream, err error) {
	var twitchResponse struct {
		Streams []twitchStream `json:"streams"`
	}

	params := url.Values{}

	if channels != nil {
		// e.g. esl_csgo => 31239503 via https://dev.twitch.tv/docs/v5/reference/users/#get-users

		channels, err = c.ChannelsToID(channels)
		if err != nil {
			return
		}

		params.Set("channel", strings.Join(channels, ","))
	}

	if game != nil {
		params.Set("game", *game)
	}

	if language != nil {
		params.Set("language", *language)
	}

	err = c.Request("/streams/", params, &twitchResponse)
	if err != nil {
		return
	}

	response = make([]Stream, 0)

	for _, stream := range twitchResponse.Streams {
		if stream.Game != "" { // For some reason sometimes the game is empty.
			response = append(response, stream.toSimplified())
		}
	}

	return
}
