package twitch

import (
	"encoding/json"
	"errors"
	"fmt"
	"io/ioutil"
	"net/http"
	"net/url"
	"strings"
)

// Client is used to request the Twitch API
type Client struct {
	// ClientID is the twitch client id used for requests. Must be set before any other calls.
	ClientID string

	// channelsToIDCache is used by ChannelsToId to cache the channels to id mapping
	channelsToIDCache map[string]string
}

// NewClient returns a new Client with the provided clientID
func NewClient(clientID string) *Client {
	return &Client{
		ClientID:          clientID,
		channelsToIDCache: make(map[string]string),
	}
}

// Request requests the give twitch api kraken endpoint.
func (c *Client) Request(endpoint string, query url.Values, output interface{}) error {
	if c.ClientID == "" {
		return errors.New("Twitch ClientID is not set")
	}

	query.Set("limit", "100")

	url := fmt.Sprintf("https://api.twitch.tv/kraken%s?%s", endpoint, query.Encode())

	req, err := http.NewRequest("GET", url, nil)
	if err != nil {
		return err
	}

	req.Header.Add("Client-ID", c.ClientID)
	req.Header.Add("Accept", "application/vnd.twitchtv.v5+json")

	resp, err := http.DefaultClient.Do(req)
	if err != nil {
		return err
	}

	defer resp.Body.Close()

	body, err := ioutil.ReadAll(resp.Body)
	if err != nil {
		return err
	}

	if resp.StatusCode != http.StatusOK {
		return fmt.Errorf("Twitch returned %s: %s", resp.Status, body)
	}

	err = json.Unmarshal(body, output)
	if err != nil {
		return err
	}

	return nil
}

// ChannelsToID translates the given channels to ids.
func (c *Client) ChannelsToID(channels []string) (ids []string, err error) {
	var unknownChannels []string

	for _, channel := range channels {
		if _, ok := c.channelsToIDCache[channel]; !ok {
			unknownChannels = append(unknownChannels, channel)
		}
	}

	if unknownChannels != nil { // we have unknown channels
		var twitchResponse struct {
			Users []struct {
				ID   string `json:"_id"`
				Name string `json:"name"`
			} `json:"users"`
		}

		params := url.Values{}
		params.Set("login", strings.Join(unknownChannels, ","))

		err = c.Request("/users", params, &twitchResponse)

		if err != nil {
			return
		}

		for _, user := range twitchResponse.Users {
			c.channelsToIDCache[user.Name] = user.ID
		}
	}

	for _, channel := range channels {
		ids = append(ids, c.channelsToIDCache[channel])
	}

	return
}
